
from copy import copy, deepcopy

from dmzb.slang.pidgin.unifier import Unifier
      
class Frame:
    counter = 0
    
    @classmethod
    def next_counter(cls):
        counter = cls.counter
        cls.counter = cls.counter + 1
        return counter
    
    def __init__(self, unificand, unifier=Unifier(), pipeline=[]):
        self.id = self.__class__.next_counter()
        self.pipeline = pipeline
        self.unificand = unificand
        self.unifier = unifier
    
    def add(self, items):
        item = items[0]
        items = items[1:]
        if items:
            self.add(items)
        self.pipeline.append(item)
        return self
        
    def clone(self):
        return Frame(copy(self.unificand), self.unifier.clone(), copy(self.pipeline))
    
    def bind(self, variable):
        if self.unificand:
            value = self.unificand[0].value
            if self.unifier.bind(variable, value):
                return self.transfer(value)
        return False
    
    def next(self):
        if self.pipeline:
            return self.pipeline.pop()
        return None
    
    def is_complete(self):
        return len(self.unificand) == 0 and len(self.pipeline) == 0
    
    def transfer(self, value):
        if self.unificand:
            if self.unificand[0].value == value:
                self.unificand = self.unificand[1:]
                self.unifier.add_token(value)
                return True
        return False
    
    def __str__(self):
        bang = '!' if self.is_complete() else ''
        nodes = [str(node) for node in self.pipeline]
        values = [item.value for item in self.unificand]
        return '%s: %s//%s%s -- %s' % (self.id, values, self.unifier, bang, nodes)

