#!/usr/local/bin/python2.7
# encoding: utf-8

import sys

from argparse import ArgumentParser
from dmzb.slang.pidgin import lexer
from dmzb.slang.pidgin.recognizer import Frame, Monitor, Recognizer
from dmzb.slang.pidgin.scanner import Scanner
from random import sample

__all__ = []

DEFAULT_INPUT = """
    channel: [list] channels.
    channel: describe :channel-spec.
    channel-spec: channel [named] <channel-name>
    channel-spec: channel number <channel-number>
    recording: [list] recordings.
    recording: describe recording [with [program] id] <program-id>.
    recording: [list] recordings marked for delete.
    recording: [list] recordings for :channel-spec.
"""
DEFAULT_SAMPLES = [
    "list channels",
    "describe recording 12",
    "describe recording with program id 12",
    "describe recording with id 12",
    "describe channel Disney",
    "describe channel named Disney",
    "describe channel number 912",
]

class UserExit(Exception):
    pass

class Lizard(Monitor):
    def __init__(self, input, samples):
        self.input = input
        self.samples = samples
    
    def run(self):
        predicates = lexer.predicates(self.input)
        sample = self.select()
        while sample:
            recognizer = Recognizer(self)
            unificand = Scanner.scan(sample)
            try:
                recognizer.search(predicates, Frame(unificand))
            except UserExit:
                pass
            sample = self.select()
    
    def select(self):
        while True:
            user = raw_input("[l/n/-/q]: ").strip()
            try:
                number = int(user)
                sample = self.samples[number]
                return sample
            except:
                pass
            if user[0] in 'Ll':
                for index, sample in enumerate(self.samples):
                    print index, sample
                print
            elif user[0] in 'Qq':
                return None
            elif user[0] == '-':
                return user[1:]
            else:
                print 'l: list samples'
                print "n (0-%s): choose sample #n" % len(self.samples)
                print "q: quit"
                print "-text: use 'text' as sample"
                print
    
    def leave(self, frame, stack):
        print "leave"
        print 'frame: ', frame
        self.leaving(stack)
    
    def leaving(self, stack):
        while True:      
            user = raw_input('c/q/S/s').strip()
            if user:
                if user[0] in 'Cc':
                    return
                elif user[0] in 'Qq':
                    raise UserExit
                elif user[0] in 'Ss':
                    self.show_stack(user[0] == 's', stack)
                else:
                    print "c: continue"
                    print "q: quit"
                    print "s: short stack"
                    print "S: full stack"
                    print
            else:
                return

    def show_stack(self, full, stack):
        if full:
            count = min(10, len(stack))
        else:
            count = len(stack)
        for index in range(count - 1):
            frame = stack[index]
            print index, 'frame:', frame
        print

    def visit(self, node, frame, stack):
        print 'visit'
        if node.is_leaf():
            flag = '*'
        elif node.is_disjunction():
            flag = '+'
        else:
            flag = '&'
        print "node[%s%s]: " % (node.__class__.__name__, flag), node
        print 'frame: ', frame
        self.visiting(stack)
    
    def visiting(self, stack):
        while True:      
            user = raw_input('c/s/S/q').strip()
            if user:
                if user[0] in 'Cc':
                    return
                elif user[0] in 'Qq':
                    raise UserExit
                elif user[0] in 'Ss':
                    self.show_stack(user[0] == 's', stack)
                else:
                    print "c: continue"
                    print "q: quit"
                    print "s: short stack"
                    print "S: full stack"
                    print
            else:
                return
    def unify(self, node, frame, unified):
        print 'unify'
        print 'node:', node
        print 'frame:', frame
        print 'unified:', unified
        self.unifying()
     
    def unifying(self):         
        while True:      
            user = raw_input('c/q').strip()
            if user:
                if user[0] in 'Cc':
                    return
                elif user[0] in 'Qq':
                    raise UserExit
                else:
                    print "c: continue"
                    print "q: quit"
                    print
            else:
                return
            
def main(argv=None): # IGNORE:C0111
    '''Command line options.'''

    if argv is None:
        argv = sys.argv
    else:
        sys.argv.extend(argv)

    input = DEFAULT_INPUT
    samples = DEFAULT_SAMPLES
    try:
        parser = ArgumentParser()
        parser.add_argument("-i", "--input", dest="input", help="pidgin source")
        parser.add_argument("-s", "--samples", dest="samples", help="test samples")

        args = parser.parse_args()

        if args.input:
            with open(args.input) as readable:
                input = readable.readlines()
        
        if args.samples:
            with open(args.samples) as readable:
                samples = readable.readlines()
        
    except Exception, e:
        indent = len(sys.argv[0]) * " "
        sys.stderr.write(sys.argv[0] + ": " + repr(e) + "\n")
        sys.stderr.write(indent + "  for help use --help")
        return 2
    
    lizard = Lizard(input, samples)
    lizard.run()
    return 0

if __name__ == "__main__":
    sys.exit(main())