
from dmzb.slang.pidgin import model
from dmzb.slang.pidgin.frame import Frame

class Monitor:
    
    def leave(self, frame, stack): pass
    def visit(self, node, frame, stack): pass
    def unify(self, node, frame, unified): pass

class Recognizer:
    def __init__(self, monitor=None):
        self.monitor = monitor
    
    def search(self, node_zero, frame_zero):
        frame_zero.add([node_zero])
        stack = [frame_zero]
        frame = None
        while stack and (not frame or not frame.is_complete()):
            frame = stack.pop()
            node = frame.next()
            if self.monitor: self.monitor.visit(node, frame, stack)
            frame.unifier.visit_node(node)
            if node.is_leaf():
                unified = self.unify(node, frame)
                if self.monitor: self.monitor.unify(node, frame, unified)
                if unified:
                    stack.append(frame)
                else:
                    frame = None
            elif node.is_disjunction():
                stack.extend([frame.clone().add([child]) for child in node.get_children()])
                frame = None
            else:
                frame.add(node.get_children())
                stack.append(frame)
                frame = None
            if self.monitor: self.monitor.leave(frame, stack)
        if frame and frame.is_complete():
            return frame.unifier
        
    @classmethod
    def recognize(cls, module, unificand):
        solver = cls()
        frame = Frame(unificand)
        result = solver.search(module, frame)
        return result

    def unify(self, node, frame):
        if not node.is_leaf():
            raise Exception("No unification for non-leaf nodes")
        elif isinstance(node, model.String):
            return frame.transfer(node.value)
        elif isinstance(node, model.Trivial):
            return True
        elif isinstance(node, model.Variable):
            return frame.bind(node)
        elif isinstance(node, model.Stop):
            return frame.is_complete()
