
import ply.lex as lex

class Scanner:

    tokens = (
        'ADDRESS',
        'NUMBER',
        'STRING',
        'URL',
        'WORD',
    )
    
    
    singleton = None
    
    @classmethod
    def the_lexer(cls):
        if not cls.singleton:
            cls.singleton = Scanner()
        return cls.singleton.lexer()
    
    @classmethod
    def scan(cls, data):
        lexer = cls.the_lexer()
        lexer.input(data)
        result = []
        while True:
            token = lexer.token()
            if not token: break
            result.append(token)
        return result

    def __init__(self):
        self.lex = None
    
    def lexer(self):
        if not self.lex:
            self.lex = lex.lex(object=self) 
        return self.lex
        
    def t_ADDRESS(self, t):
        r'([a-z0-9_\.-]+)@([\da-z\.-]+)\.([a-z\.]{2,6})'
        return t
    
    def t_NUMBER(self, t):
        r'[0-9]+'
        return t
    
    def t_STRING(self, t):
        r'\"(\\.|[^\"])*\"'
        return t
    
    def t_WORD(self, t):
        r'[a-zA-Z][a-zA-Z0-9\-\_]*'
        return t
    
    def t_URL(self, t):
        r"(?i)\b((?:[a-z][\w-]+:(?:/{1,3}|[a-z0-9%])|www\d{0,3}[.]|[a-z0-9.\-]+[.][a-z]{2,4}/)(?:[^\s()<>]+|\(([^\s()<>]+|(\([^\s()<>]+\)))*\))+(?:\(([^\s()<>]+|(\([^\s()<>]+\)))*\)|[^\s`!()\[\]{};:'\".,<>?]))"
        return t
    
    def t_error(self, t):
        raise Exception("Illegal character '%s'" % t.value[0])
    
    t_ignore  = ' \t'


