
from copy import copy, deepcopy

from .model import Predicate
      
class Unifier:
    def __init__(self, tokens=[], variables={}, predicate=None):
        self.tokens = tokens
        self.variables = variables
        self.predicate = predicate
    
    def add_token(self, token):
        self.tokens.append(token)
    
    def bind(self, variable, value):
        if variable.name in self.variables:
            return self.variables[variable.name] == value
        else:
            self.variables[variable.name] = value
            return True
    
    def clone(self):
        return Unifier(copy(self.tokens), deepcopy(self.variables), self.predicate)
    
    def has_subsequence(self, subseq):
        return any(self.tokens[pos:pos + len(subseq)] == subseq for pos in range(0, len(self.tokens) - len(subseq) + 1))
    
    def visit_node(self, node):
        if isinstance(node, Predicate) and not self.predicate:
            self.predicate = node
    
    def __str__(self):
        return "%s(%s)" % (self.tokens, len(self.variables))