
import unittest

from dmzb.slang.pidgin import model
from dmzb.slang.pidgin import lexer

class TestLexer(unittest.TestCase):

    def test_bar(self):
        tokens = lexer.Lexer.tokenize("this: that|'string'")
        self.assertIsNotNone(tokens, "Bar test yielded no tokens")

    def test_brackets(self):
        tokens = lexer.Lexer.tokenize("this: <that> ['string']")
        self.assertIsNotNone(tokens, "String test yielded no tokens")

    def test_escaped(self):
        tokens = lexer.Lexer.tokenize("this: <that> ['string with \\'spaces\\']")
        self.assertIsNotNone(tokens, "String test yielded no tokens")

    def test_simple(self):
        tokens = lexer.Lexer.tokenize("this: that the-other")
        self.assertIsNotNone(tokens, "Simple test yielded no tokens")

    def test_string(self):
        tokens = lexer.Lexer.tokenize("this: that 'string'")
        self.assertIsNotNone(tokens, "String test yielded no tokens")

    def test_stop(self):
        tokens = lexer.Lexer.tokenize("this: that 'string'.")
        self.assertIsNotNone(tokens, "Stop test yielded no tokens")

class TestParser(unittest.TestCase):
    
    def check(self, result):
        self.assertIsNotNone(result, 'Simple test returned None')
        self.assertIsInstance(result, model.Predicate, 'Simple test returned class: %s' % result.__class__)
        self.assertIsNotNone(result.clauses, 'Simple test pattern has no clauses')
    
    def test_alternate(self):
        result = lexer.parse("s1: 'this' | that")
        self.check(result)

    def test_optional(self):
        result = lexer.parse("s1: ['this' thing] that")
        self.check(result)

    def test_variable(self):
        result = lexer.parse("s1: 'this' <that>")
        self.check(result)

    def test_string(self):
        result = lexer.parse("s1: 'this'")
        self.check(result)

    def test_stop(self):
        result = lexer.parse("s1: 'this'.")
        self.check(result)

class TestGraph(unittest.TestCase):
    def test_graph(self):
        data = '''
            channel: [list] channels.
            channel: describe :channel-spec.
            channel-spec: channel [named] <channel-name>
            channel-spec: channel number <channel-number>
        '''
        result = lexer.predicates(data)
        self.assertIsInstance(result, model.Module, 'Module result is not a predicates')
        self.assertIsInstance(str(result), str, 'No string representation of predicates')

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()